use strict;
use Test::Simple tests => 11;

sub mysystem($$) {
    my ($cmd, $desc) = @_;
    if (system $cmd) {
	my ($subcmd) = split ' ', $cmd;
	ok(0, "$desc: $subcmd: ".($? >> 8));
	return 0;
    }
    return 1;
}

sub output_is {
    my ($code, $out, $desc) = @_;
    unless ($desc) {
	(undef, my $file, my $line) = caller;
	$desc = "($file line $line)";
    }
    open(O, "| perl prd-perl6.pl --batch --imc > a.imc 2>/dev/null") or die $!;
    print O $code;
    unless (close O) {
	ok(0, "$desc: parse: $!");
	return;
    }
    (mysystem("../languages/imcc/imcc a.imc a.pasm 2>/dev/null", $desc)
     && mysystem("perl ../assemble.pl a.pasm > a.pbc", $desc)
     && mysystem("../parrot a.pbc > a.output", $desc))
	or return 0;
    open(I, 'a.output');
    my $result = join '', <I>;
    ok($out eq $result, $desc);
}

##############################
output_is(<<'CODE', <<'OUT', "Basic hello.");
sub main() {
    print1("Hello, " _ "world");
}
CODE
Hello, world
OUT

##############################
output_is(<<'CODE', <<'OUT', "Binary math ops");
sub main() {
    print1(2 + 3);
    print1(2 - 3);
    print1(2 * 3);
    print1(6 / 2);
    print1(2 % 3);
    print1(2 _ 3);
    print1(2 ** 3);
}
CODE
5
-1
6
3.000000
2
23
8
OUT

##############################
output_is(<<'CODE', <<'OUT', "Binary bit ops");
sub main() {
    print1(2 << 3);
    print1(32 >> 3);
    print1(32 | 3);
    print1(31 & 3);
    print1(10 ~ 12);		# 1010 ~ 1100 -> 0110 == 6
}
CODE
16
4
35
3
6
OUT

##############################
output_is(<<'CODE', <<'OUT', "Binary logical ops");
sub main() {
    print1(2 || 3);
    print1(0 || 2);
    print1(2 || 0);
    print1(0 || 0);
    print1(2 && 3);
    print1(2 && 0);
    print1(0 && 2);
    print1(0 && 0);
    print1(2 ~~ 3);
    print1(2 ~~ 0);
    print1(0 ~~ 2);
    print1(0 ~~ 0);
    my $x;
    print1($x // 0);
    print1(0  // $x);
    print1($x // $x);
    print1(2  // 3);
}

CODE
2
2
2
0
3
0


0
1
1
0
0
0

2
OUT

##############################
output_is(<<'CODE', <<'OUT', "equality");
sub main() {
    my ($x, $y) = (1, 2);
    my ($a, $b) = ('1', '2');
    unless $a eq $x == $a {
    } elsunless $a ne $b != $x {
    } elsunless $a + $x eq $b == $y {
    } else {
	print1("ok");
    }
}

CODE
ok
OUT

##############################
output_is(<<'CODE', <<'OUT', "gt/lt");
sub main() {
    my ($x, $y, $z, $w) = (1, '2', 21, '122');
    unless $x < $y < $z < $w {
    } elsunless $x lt $w lt $y lt $z {
    } elsunless $w > $z > $y > $x {
    } elsunless $z gt $y gt $w gt $x {
    } elsunless $z gt $w > $z {
    } else {
	print1("ok");
    }
}

CODE
ok
OUT

##############################
output_is(<<'CODE', <<'OUT', "subroutine call");
sub _fact($tot, $max, $n) {
    if $n > $max {
	print1($max _ "! = " _ $tot);
    } else {
	_fact $tot * $n, $max, $n + 1;
    }
}

sub fact($n) {
    unless 0 <= $n < 20 {
	print1("Sorry, can't take " _ $n _ " factorial");
    } else {
	_fact 1, $n, 1
    }
}

sub main() {
    fact 12;
    fact 14;
    fact -1;
    fact 0;
    fact 24;
}

CODE
12! = 479001600
14! = 1278945280
Sorry, can't take -1 factorial
0! = 1
Sorry, can't take 24 factorial
OUT

##############################
output_is(<<'CODE', <<'OUT', "elsunless");
sub main() {
    if 2 > 1 {
	print1("ok")
    } else {
	print1("not ok")
    }
    unless 1 > 2 {
	print1("ok")
    } else {
	print1("not ok")
    }
    unless 2 > 1 {
	print1("not ok")
    } elsunless 3 > 2 {
	print1("not ok")
    } elsif "yep" {
	print1("ok")
    } else {
	print1("not ok")
    }
}

CODE
ok
ok
ok
OUT

##############################
output_is(<<'CODE', <<'OUT', 'Ternary');
sub main() {
    my ($x, $y, $z) = (1,2,3);
    my ($a, $b);
    $a = ($x ?? $b :: $y) = $y + 3;
    print1(0 ?? "bad" :: "ok");
    print1(1 ?? "ok" :: "bad");
    print1($a);
    print1($b);
    print1($y);
    print1($x ?? $b - $a ?? "bad" :: "ok" :: "foo");
}
CODE
ok
ok
5
5
2
ok
OUT

##############################
output_is(<<'CODE', <<'OUT', 'Arrays');
sub main() {
    my @a = (1,2,3,4);
    my @b = @a[0,2];
    print1(@b[0] _ ', ' _ @b[1]);
    print1(@a[0] _ ', ' _ @a[2]);
    print1(@a[1] _ ', ' _ @a[3]);
}

CODE
1, 3
1, 3
2, 4
OUT

##############################
output_is(<<'CODE', <<'OUT', 'Hyper 1');
sub parray(@y) {
    print1('(' _ @y[0] _ ', ' _ @y[1] _ ')');
}
sub main() {
    my @a = (1,2);
    my @x = (3,4);
    my @b = (5,6);
    my $i = 2;
    parray @a ^* @x ^+ @b;
    parray $i ^* @x ^+ @b;
    parray @a  * @x ^+ @b;
    parray 2 * 3 ^+ @b;
    print1(@a ^* @x  + @b);
# IMCC clobbers too many registers with this:
#     @y = @a ^<< @a;
#     print1('(' _ @y[0] _ ', ' _ @y[1] _ ')');
}
CODE
(8.000000, 14.000000)
(11.000000, 14.000000)
(9.000000, 10.000000)
(11.000000, 12.000000)
4
OUT

<<'DONT';
##############################
output_is(<<'CODE', <<'OUT', 'Hyper 2');
sub main() {
    my @a = (0,1,0,1);
    my @b = (0,0,1,1);
    my @y = @a ^~~ @b;
    print1('('_@y[0]_', '_@y[1]_', '_@y[2]_', '_@y[3]_')');
}

CODE
(0, 1, 1, 0)
OUT

DONT
